' Testing interface to ADS1248 ADC with PGA and bias supply
' Raymond Allen 2019
' MIT license
'  This code reads and displays 24-bit data on the default input pins and gain set to 16x
'  Converts at the default rate of 5 SPS.  We are reading data from chip at lower rate, so don't have to monitor data ready pin.

CON    'Your standard clock&crystal settings
    _clkmode = xtal1 + pll16x                           
    _xinfreq = 5_000_000

CON  'ADS1248 PINS
  ACLK_Pin=3    'ADC clock pin
  ACSN_Pin=6    'ADC chip select
  AMISO_Pin=5   'ADC Data ready and DO
  AMOSI_Pin=4   'ADC DI
  AReset_Pin=2  'ADC reset
  AStart_Pin=7  'ADC start

CON 'Note: This pin is specific to this test application, not needed by ADS1248 in general
  TCEN_Pin=1    'thermocouple current enable

   
        
OBJ  'required objects
  ser     :       "fullduplexserial"   'for debugging
  spi     :       "SPI_ASM_RJA2" 'SPI driver (fixed by RJA for 32 bits)

PUB Main | i,j,k
    'Turn on thermocouple current(again needed by this particular board, not by ADS1248 in general)
    outa[tcen_pin]~~
    dira[tcen_pin]~~

    ser.start(31,30,0,115200)     'serial for debugging only

    'Init ADC pins    
    outa[ACSN_Pin]~~      'Take control of CS   
    dira[ACSN_Pin]~~    
    outa[AReset_Pin]~~      'Take control of reset  
    dira[AReset_Pin]~~
    outa[AStart_Pin]~       'Take control of start 
    dira[AStart_Pin]~~
    dira[AMISO_Pin]~        'DRDY/DO is input for now
    waitcnt(cnt+clkfreq)
    
    'reset ADC
    outa[AReset_Pin]~
    waitcnt(cnt+clkfreq/1000) 
    outa[AReset_Pin]~~
    'Select chip  - Note:  Just leaving CS always low.  May tie to ground in future.
    outa[ACSN_Pin]~
    waitcnt(cnt+clkfreq/10)

    'Start SPI
    spi.start(30,0) 'delay=15, start with clock low


    outa[AStart_Pin]~~   
    waitcnt(cnt+clkfreq/10)
     '{   'send reset command
    repeat 1
      waitcnt(cnt+clkfreq/10)
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $06)   '$06==reset
      ser.tx(13)
      ser.tx("X")
      ser.tx(13)
      waitcnt(cnt+clkfreq)
     '}

    '{   'stop continuous read mode
    repeat 1
      waitcnt(cnt+clkfreq/100)
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $16)   'SDATAC
      ser.tx(13)
      ser.tx(".")
      ser.tx(13)
      waitcnt(cnt+clkfreq)        
    '}
   

    '{   'enable internal reference
    repeat 1
      waitcnt(cnt+clkfreq/10)
       spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $42)   'write register
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $00)   '1 byte
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, %00110000)   '1 byte 
      ser.tx(13)
      ser.tx(".")
      ser.tx(13)
      waitcnt(cnt+clkfreq)
     '} 

     '{   'enable   DRDY MODE
    repeat 1
      waitcnt(cnt+clkfreq/10)
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $4A)   'write register
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $00)   '1 byte
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, %00001000)   '1 byte 
      ser.tx(13)
      ser.tx(".")
      ser.tx(13)
      waitcnt(cnt+clkfreq)        
     '}

    '{   'Set PGA gain
    repeat 1
      waitcnt(cnt+clkfreq/10)
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $43)   'write register #3
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $00)   '1 byte
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, %01000000)   '%01000000==16 gain at 5 SPS 
      ser.tx(13)
      ser.tx("o")
      ser.tx(13)
      waitcnt(cnt+clkfreq)        
     '}

    

    '{    'read 11 registers, starting at #2
    repeat 1
      waitcnt(cnt+clkfreq/100)  
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, $22)   'read register
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, SPI#MSBFIRST, 8, 10)   '11 bytes to read in
      waitcnt(cnt+clkfreq/100)
      outa[AMOSI_Pin]~
      dira[AMOSI_Pin]~~
      repeat 11
        i:=spi.SHIFTIN(AMISO_Pin, ACLK_Pin, spi#MSBPOST, 8)
        ser.bin(i,8)
        ser.tx(13)
      ser.tx(13)
      dira[AMOSI_Pin]~
      waitcnt(cnt+clkfreq)        
     '}

    'Finally, read in data (note:  For now, just reading continously and slower that 5 Samples per second so don't have to monitor DRDY pin).   
    repeat
      waitcnt(cnt+clkfreq/4)   'This makes sure reading slower than 5 samples per second (the default config)
      i:=cnt  'Note:  In future, may use ASTART pin to initiate conversion and DRDY pin to know when conversion complete
      i:=cnt-i 'get click ticks to convert (future use, as mentioned in line above)
      ser.tx(13)
      ser.dec(i)
      ser.tx(13)
      waitcnt(cnt+clkfreq/100)
      spi.SHIFTOUT(AMOSI_Pin, ACLK_Pin, spi#MSBFIRST, 8, $12)   'RDATA=read data once
      outa[AMOSI_Pin]~~  'clock out NOP, while reading in data
      dira[AMOSI_Pin]~~ 
      i:=spi.SHIFTIN(AMISO_Pin, ACLK_Pin, spi#MSBPOST, 24)
      dira[AMOSI_Pin]~ 
      'Show 24-bit data
      ser.bin(i,24)
      ser.tx(13)
      ser.tx(13)

        
      
  



